package cn.iocoder.yudao.module.member.service.user;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.MemberRegEvent;
import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.spring.enums.CommonStatusEnum;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.module.ecw.api.customer.CustomerApi;
import cn.iocoder.yudao.module.ecw.api.internalMessage.ClientInternalMessageApi;
import cn.iocoder.yudao.module.ecw.api.internalMessage.dto.InternalMessageCreateDto;
import cn.iocoder.yudao.module.ecw.api.paramValid.ParamValidatorApi;
import cn.iocoder.yudao.module.ecw.enums.CustomerStatusEnum;
import cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.member.api.user.dto.UserSelectReqVO;
import cn.iocoder.yudao.module.member.controller.admin.user.vo.UserBackVO;
import cn.iocoder.yudao.module.member.controller.admin.user.vo.UserQueryVO;
import cn.iocoder.yudao.module.member.controller.app.auth.vo.AppEmailUpdateReqVO;
import cn.iocoder.yudao.module.member.controller.app.user.vo.AppUserUpdateMobileReqVO;
import cn.iocoder.yudao.module.member.convert.user.UserConvert;
import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;
import cn.iocoder.yudao.module.member.dal.mysql.user.MemberUserMapper;
import cn.iocoder.yudao.module.member.dal.mysql.userCardAuth.UserCardAuthMapper;
import cn.iocoder.yudao.module.member.dal.mysql.userEnterpriseAuth.UserEnterpriseAuthMapper;
import cn.iocoder.yudao.module.member.enums.UserOperationLogTypeEnum;
import cn.iocoder.yudao.module.member.enums.UserQueryAuthTypeEnum;
import cn.iocoder.yudao.module.member.service.userOperationLog.UserOperationLogService;
import cn.iocoder.yudao.module.member.util.MemberUserCodeUtils;
import cn.iocoder.yudao.module.member.vo.member.MemberUpdateReqVO;
import cn.iocoder.yudao.module.member.vo.userOperationLog.UserOperationLogCreateReqVO;
import cn.iocoder.yudao.module.product.service.coupon.CouponService;
import cn.iocoder.yudao.module.system.api.sms.SmsCodeApi;
import cn.iocoder.yudao.module.system.api.sms.dto.code.SmsCodeUseReqDTO;
import cn.iocoder.yudao.module.system.enums.sms.SmsSceneEnum;
import com.alibaba.excel.util.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.annotations.VisibleForTesting;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.context.ApplicationContext;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.io.InputStream;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.util.servlet.ServletUtils.getClientIP;
import static cn.iocoder.yudao.module.infra.enums.ErrorCodeConstants.GET_LOCK_FAILED;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.*;

/**
 * 会员 User Service 实现类
 *
 * @author 捷道源码
 */
@Service
@Valid
@Slf4j
public class MemberUserServiceImpl implements MemberUserService {

    @Resource
    private MemberUserMapper memberUserMapper;

    @Resource
    private UserCardAuthMapper userCardAuthMapper;

    @Resource
    private UserEnterpriseAuthMapper userEnterpriseAuthMapper;
    @Resource
    private ClientInternalMessageApi clientInternalMessageApi;
    @Resource
    private FileApi fileApi;
    @Resource
    private SmsCodeApi smsCodeApi;

    @Resource
    private PasswordEncoder passwordEncoder;

    @Resource
    private CustomerApi customerApi;

    @Resource
    private CouponService couponService;

    @Resource
    private UserOperationLogService userOperationLogService;

    @Resource
    private ApplicationContext applicationContext;

    @Resource
    private ParamValidatorApi paramValidatorApi;

    @Resource
    private RedisHelper redisHelper;

    @Resource
    private RedissonClient redissonClient;

    @Override
    public MemberUserDO getUserByMobile(String mobile) {
        return memberUserMapper.selectByMobile(mobile);
    }

    @Override
    public MemberUserDO checkPhoneUnique(String areaCode, String mobile) {
        //首先查号码是否存在
        MemberUserDO memberUserDO = memberUserMapper.selectByMobile(areaCode, mobile);
        if (memberUserDO != null) {
            return memberUserDO;
        }

        if (mobile.startsWith("0")) {
            //如果0开头,查去掉0的号码是否存在
            String mobile2 = mobile.substring(1);
            memberUserDO = memberUserMapper.selectByMobile(areaCode, mobile2);
            return memberUserDO;
        } else {
            //如果非0开头,查开头补0的号码是否存在
            String mobile2 = "0" + mobile;
            memberUserDO = memberUserMapper.selectByMobile(areaCode, mobile2);
            return memberUserDO;
        }
    }

    public MemberUserDO getUserByMobile(String areaCode, String mobile) {
        return memberUserMapper.selectByMobile(areaCode, mobile);
    }

    @Override
    public MemberUserDO getUserByEmail(String email) {
        return memberUserMapper.selectByEmail(email);
    }

    @Override
    public MemberUserDO createUserIfAbsent(String mobile, String areaCode, String registerIp, Integer platform) {

        try {
            String mobileCode = areaCode + StrUtil.COLON + mobile;
            paramValidatorApi.validatorMobile(mobileCode);
//            String phone = PhoneUtil.formatPhone(mobileCode);
//            if (StringUtils.isNotBlank(phone)) {
//                mobile = phone;
//            }
        } catch (Exception e) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.MOBILE_FORMAT_ERROR);
        }
        // 用户已经存在
        MemberUserDO user = checkPhoneUnique(areaCode, mobile);
        if (user != null) {
            return user;
        }
        String englishName = maskMobileNumber(mobile);
        // 用户不存在，则进行创建
        user = this.createUser(mobile, registerIp, null, areaCode, englishName, null, platform);

        //记录日志
        UserOperationLogCreateReqVO userOperationLogCreateReqVO = new UserOperationLogCreateReqVO();
        userOperationLogCreateReqVO
                .setUserId(user.getId())
                .setType(UserOperationLogTypeEnum.REGIST.getValue())
                .setTitle(UserOperationLogTypeEnum.REGIST.getDesc())
                .setIp(getClientIP())
                .setContent("手机验证码注册成功");

        userOperationLogService.createUserOperationLog(userOperationLogCreateReqVO);

        return user;
    }

    public String maskMobileNumber(String mobile) {
        final String mask = "*******";
        mobile = mobile == null ? mask : mobile;
        final int lengthOfMobileNumber = mobile.length();
        if (lengthOfMobileNumber > 2) {
            final int maskLen = Math.min(Math.max(lengthOfMobileNumber / 2, 2), 6);
            final int start = (lengthOfMobileNumber - maskLen) / 2;
            return mobile.substring(0, start) + mask.substring(0, maskLen) + mobile.substring(start + maskLen);
        }
        return mobile;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public MemberUserDO createUser(String mobile, String registerIp, String password, String areacode, String englishName, String referralCode, Integer registerPlatform) {
        // 生成密码
       /* if (StrUtil.isBlank(password)) {
            password = IdUtil.fastSimpleUUID();
        }*/
        // 插入用户
        MemberUserDO user = new MemberUserDO();
        user.setMobile(mobile);
        user.setStatus(CommonStatusEnum.ENABLE.getStatus()); // 默认开启
        if (password != null) {
            user.setPassword(passwordEncoder.encode(password)); // 加密密码
        }
        user.setRegisterIp(registerIp);
        user.setAreaCode(areacode);
        if (StringUtils.isEmpty(englishName)) {
            englishName = maskMobileNumber(mobile);
        }
        user.setEnglishName(englishName);
        user.setNickname(englishName);
        user.setCode(getNextMemberCode());
        user.setReferralCode(referralCode);
        user.setRegisterPlatform(registerPlatform);
        memberUserMapper.insert(user);
        //站内信
        InternalMessageCreateDto dto = InternalMessageCreateDto.builder().toIdList(Arrays.asList(user.getId()))
                .fromId("")
                .sendTime(new Date())
                .title(I18nMessage.getMessage("app.message.reg.success.title"))
                .content(I18nMessage.getMessage("app.message.reg.success.content"))
                .type(3).build();
        clientInternalMessageApi.createInternalMessage(dto);

        //关联或创建后端客户
        customerApi.associateCustomerAuto(areacode, mobile, user.getId(), englishName);

        applicationContext.publishEvent(new MemberRegEvent(user.getId()));

        //获取优惠卷
        couponService.registerGiveCoupon(user.getId());

        return user;
    }

    @Override
    public void updateUserLogin(Long id, String loginIp) {
        memberUserMapper.updateById(new MemberUserDO().setId(id)
                .setLoginIp(loginIp).setLoginDate(new Date()));
    }

    @Override
    public MemberUserDO getUser(Long id) {
        return memberUserMapper.selectById(id);
    }

    @Override
    public MemberUserDO info(Long id) {
        // 一个会员可能绑定了多个用户
        return memberUserMapper.info(id).get(0);
    }

    @Override
    public List<MemberUserDO> selectList(Collection<Long> ids) {
        LambdaQueryWrapperX<MemberUserDO> wrapper = new LambdaQueryWrapperX<MemberUserDO>()
                .eq(MemberUserDO::getStatus, CommonStatusEnum.ENABLE.getStatus())
                .inIfPresent(MemberUserDO::getId, ids)
                .orderByDesc(MemberUserDO::getId);

        return memberUserMapper.selectList(wrapper);
    }

    @Override
    public PageResult<MemberUserDO> selectList(UserSelectReqVO userSelectReqVO) {

        IPage<MemberUserDO> mpPage = MyBatisUtils.buildPage(userSelectReqVO);

        String searchKey = userSelectReqVO.getSearchKey();

        LambdaQueryWrapper<MemberUserDO> wrapper = new LambdaQueryWrapper<MemberUserDO>()
                .eq(MemberUserDO::getStatus, CommonStatusEnum.ENABLE.getStatus())
                .and(StringUtils.isNotEmpty(searchKey),
                        i -> i.like(MemberUserDO::getNickname, searchKey).or().like(MemberUserDO::getMobile, searchKey))
                .orderByDesc(MemberUserDO::getId);

        memberUserMapper.getPage(mpPage, wrapper);

        return new PageResult<>(mpPage.getRecords(), mpPage.getTotal());
    }

    @Override
    public void updateUserNickname(Long userId, String nickname) {
        MemberUserDO user = this.checkUserExists(userId);
        // 仅当新昵称不等于旧昵称时进行修改
        if (nickname.equals(user.getNickname())) {
            return;
        }
        MemberUserDO userDO = new MemberUserDO();
        userDO.setId(user.getId());
        userDO.setNickname(nickname);
        memberUserMapper.updateById(userDO);
    }

    @Override
    public String updateUserAvatar(Long userId, InputStream avatarFile) throws Exception {
        this.checkUserExists(userId);
        // 创建文件
        String avatar = fileApi.createFile(IoUtil.readBytes(avatarFile));
        // 更新头像路径
        memberUserMapper.updateById(MemberUserDO.builder().id(userId).avatar(avatar).build());
        return avatar;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateUserMobile(Long userId, AppUserUpdateMobileReqVO reqVO) {
        // 检测用户是否存在
        checkUserExists(userId);

        //检测用户密码是否正确
        checkOldPassword(userId, reqVO.getPassword());

       /* // 校验旧手机和旧验证码
        smsCodeApi.useSmsCode(new SmsCodeUseReqDTO().setMobile(reqVO.getOldMobile()).setCode(reqVO.getOldCode())
                .setScene(SmsSceneEnum.MEMBER_UPDATE_MOBILE.getScene()).setUsedIp(getClientIP()));*/
        // 使用新验证码
        smsCodeApi.useSmsCode(new SmsCodeUseReqDTO().setMobile(reqVO.getMobile()).setCode(reqVO.getCode())
                .setScene(SmsSceneEnum.MEMBER_UPDATE_MOBILE.getScene()).setUsedIp(getClientIP()));

        // 更新用户手机
        memberUserMapper.updateById(MemberUserDO.builder().id(userId).mobile(reqVO.getMobile()).areaCode(reqVO.getAreaCode()).build());
    }

    @VisibleForTesting
    public MemberUserDO checkOldPassword(Long id, String oldPassword) {
        MemberUserDO user = memberUserMapper.selectById(id);
        if (user == null) {
            throw exception(USER_NOT_EXISTS);
        }
        // 参数：未加密密码，编码后的密码
        if (!passwordEncoder.matches(oldPassword, user.getPassword())) {
            throw exception(USER_PASSWORD_FAILED);
        }
        return user;
    }

    @VisibleForTesting
    public MemberUserDO checkUserExists(Long id) {
        if (id == null) {
            return null;
        }
        MemberUserDO user = memberUserMapper.selectById(id);
        if (user == null) {
            throw exception(USER_NOT_EXISTS);
        }
        return user;
    }

    @Override
    public PageResult<MemberUserDO> getUserPage(UserQueryVO query, PageVO page) {
        IPage<MemberUserDO> mpPage = MyBatisUtils.buildPage(page);
        String searchKey = query.getSearchKey();
        QueryWrapper<MemberUserDO> wrapper = getMemberUserDOQueryWrapper(query, searchKey);
        long total = memberUserMapper.memberUserCount(wrapper);
        int start = (page.getPage() - 1) * page.getRows();
        int size = page.getRows();
        wrapper.last(String.format("limit %s, %s", start, size));
        // TODO sql待优化,用户信息需要屏蔽验证权限才返回的数据, 嵌套查询优化
        List<MemberUserDO> list = memberUserMapper.getList(wrapper);
        list = list.stream().peek(m -> {
            m.setPassword(null);
            m.setControlPassword(null);
            m.setBackLetterImg(null);
        }).collect(Collectors.toList());
        return new PageResult<>(list, total, mpPage.getSize(), page.getPage(), (total + mpPage.getSize() - 1) / mpPage.getSize());
    }

    @Override
    public PageResult<MemberUserDO> getWaitApproveUserPage(UserQueryVO query, PageVO page) {
        IPage<MemberUserDO> mpPage = MyBatisUtils.buildPage(page);

        String searchKey = query.getSearchKey();

        QueryWrapper<MemberUserDO> wrapper = getMemberUserDOQueryWrapper(query, searchKey);

        memberUserMapper.getWaitApproveUserPage(mpPage, wrapper);
        return new PageResult<>(mpPage.getRecords(), mpPage.getTotal());
    }

    private QueryWrapper<MemberUserDO> getMemberUserDOQueryWrapper(UserQueryVO query, String searchKey) {
        QueryWrapper<MemberUserDO> wrapper = new QueryWrapper<MemberUserDO>()
                .eq("deleted", false)
                .and(
                        StringUtils.isNotEmpty(searchKey),
                        i -> i.like("nickname", searchKey)
                                .or()
                                .like("mobile", searchKey)
                                .or()
                                .like("identity_name", searchKey)
                                .or()
                                .like("enterprise_name", searchKey)
                );
        if (query.getBeginCreateTime() != null && query.getEndCreateTime() != null) {
            wrapper.between("create_time", query.getBeginCreateTime(), query.getEndCreateTime());
        }
        if (StringUtils.isNotBlank(query.getCustomerCode())) {
            wrapper.like("customerCode", query.getCustomerCode());
        }
        if (StringUtils.isNotBlank(query.getMemberCode())) {
            wrapper.like("code", query.getMemberCode());
        }
        //成交状态
        Boolean isDeal = query.getIsDeal();
        wrapper.apply(isDeal != null, "customer_status " + (Boolean.TRUE.equals(isDeal) ? " = 3 " : " != 3"), null);

        //认证状态
        Integer authType = query.getAuthType();
        //未认证
        wrapper.apply(UserQueryAuthTypeEnum.UNAUTHENTICATED.getValue().equals(authType), "identity_audit_status is null and enterprise_audit_status is null", null);
        //身份认证
        wrapper.apply(UserQueryAuthTypeEnum.IDENTIFICATION_AUTHENTICATED.getValue().equals(authType), "identity_audit_status is not null", null);
        //企业认证
        wrapper.apply(UserQueryAuthTypeEnum.ENTERPRISE_AUTHENTICATED.getValue().equals(authType), "enterprise_audit_status is not null", null);
        //双认证
        wrapper.apply(UserQueryAuthTypeEnum.BOTH_AUTHENTICATED.getValue().equals(authType), "identity_audit_status is not null and enterprise_audit_status is not null", null);

        //审核状态
        Integer auditStatus = query.getAuditStatus();
        if (auditStatus != null) {

            if (UserQueryAuthTypeEnum.IDENTIFICATION_AUTHENTICATED.getValue().equals(authType)) {
                wrapper.apply(true, "identity_audit_status = {0}", auditStatus);
            } else if (UserQueryAuthTypeEnum.ENTERPRISE_AUTHENTICATED.getValue().equals(authType)) {
                wrapper.apply(true, "enterprise_audit_status = {0}", auditStatus);
            } else if (UserQueryAuthTypeEnum.BOTH_AUTHENTICATED.getValue().equals(authType)) {
                wrapper.apply(true, "identity_audit_status = {0} and enterprise_audit_status = {1}", auditStatus, auditStatus);
            } else {
                //认证类型不传的情况下，只需要判断审核状态即可
                if (auditStatus == 0) {
                    wrapper.apply(true, "identity_audit_status is null and enterprise_audit_status is null", auditStatus, auditStatus);
                } else {
                    wrapper.apply(true, "identity_audit_status = {0} or enterprise_audit_status = {1}", auditStatus, auditStatus);
                }
            }
        }

        wrapper.orderByDesc("id");
        return wrapper;
    }

    @Override
    public List<MemberUserDO> getUserList(UserQueryVO query) {

        String searchKey = query.getSearchKey();

        QueryWrapper<MemberUserDO> wrapper = getMemberUserDOQueryWrapper(query, searchKey);

        return memberUserMapper.getList(wrapper);
    }

    @Override
    public int updateById(MemberUserDO memberUserDO) {
        return memberUserMapper.updateById(memberUserDO);
    }

    @Override
    public void deleteById(Long id) {
        this.checkUserExists(id);

        memberUserMapper.deleteById(id);
    }

    @Override
    public void updateUser(MemberUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateUserExists(updateReqVO.getId());
        // 更新
        MemberUserDO updateObj = UserConvert.INSTANCE.convert(updateReqVO);
        memberUserMapper.updateById(updateObj);
    }

    @Override
    public void updateNewUserAvatar(Long loginUserId, String avatar) {
        memberUserMapper.updateById(MemberUserDO.builder().id(loginUserId).avatar(avatar).build());
    }

    @Override
    public void updateEmail(Long loginUserId, AppEmailUpdateReqVO reqVO) {
        MemberUserDO user = memberUserMapper.selectById(loginUserId);
        if (user == null) {
            throw exception(USER_NOT_EXISTS);
        }
        //todo 验证码逻辑判断
        if (!"9999".equals(reqVO.getCode())) {
            throw exception(EMAIL_CODE_ERROR);
        }

        memberUserMapper.updateById(MemberUserDO.builder().id(loginUserId).email(reqVO.getEmail()).build());
    }

    @Override
    public Integer appCargoControlCount(Long userId, String phone, Integer type) {
        return memberUserMapper.appCargoControlCount(userId, phone, type);
    }

    @Override
    public Integer appIssuedOrderCount(String mobile, Long userId, Collection<Long> customerContactsIds) {
        return memberUserMapper.appIssuedOrderCount(mobile, userId, customerContactsIds);
    }

    @Override
    public Integer appReceivedOrderCount(String phone, Collection<Long> customerContactsIds) {
        return memberUserMapper.appReceivedOrderCount(phone, customerContactsIds);
    }

    @Override
    public Integer appAllOrderCount(Long userId, String phone, Collection<Long> customerContactsIds) {
        return memberUserMapper.appAllOrderCount(userId, phone, customerContactsIds);
    }

    @Override
    public List<UserBackVO> get7daysCount() {
        Date date = new Date();
        Date queryDate = DateUtils.addDate(date, Calendar.DATE, -7);
        return memberUserMapper.get7daysCount(DateUtil.format(queryDate, DateUtils.FORMAT_YEAR_MONTH_DAY));
    }

    @Override
    public void updateMemberUserDealStatus(Long customerId, Integer customerStatus) {
        Boolean isDeal = CustomerStatusEnum.DEAL.getValue().equals(customerStatus);
        memberUserMapper.updateMemberUserDealStatus(customerId, customerStatus, isDeal);
    }

    private void validateUserExists(Long id) {
        if (memberUserMapper.selectById(id) == null) {
            throw exception(USER_NOT_EXISTS);
        }
    }

    @Override
    public void memberCancellation(Long userId) {
        // 删除用户
        memberUserMapper.deleteById(userId);
        // 清除会员绑定客户信息
        memberUserMapper.unbindingCustomer(userId);
    }

    @Override
    public List<MemberUserDO> getByCursor(Long startUserId, long limit) {
        if (startUserId == null) {
            return Collections.emptyList();
        }
        LambdaQueryWrapper<MemberUserDO> wrapper = Wrappers.lambdaQuery();
        wrapper.gt(MemberUserDO::getId, startUserId);
        wrapper.orderByAsc(MemberUserDO::getId);
        wrapper.last("limit " + limit);
        return memberUserMapper.getList(wrapper);
    }

    @Override
    public void historyCodeFlush() {
        Long startUserId = 0L;
        long limit = 500;
        memberUserMapper.clearMemberCode();
        redisHelper.delete("memberCode:max:number");
        while (true) {
            List<MemberUserDO> result = this.getByCursor(startUserId, limit);
            if (CollectionUtils.isEmpty(result)) {
                break;
            }
            result.forEach(memberUserDO -> {
                String memberCode = getNextMemberCode();
                memberUserMapper.updateMemberCodeById(memberCode, memberUserDO.getId());
            });
            startUserId = result.stream().mapToLong(MemberUserDO::getId).max().orElse(Long.MAX_VALUE);
        }
    }

    private String getNextMemberCode() {
        String key = "memberCode:max:number";
        String incrIfExistScript = "if redis.call(\"exists\", KEYS[1]) == 1 then\n" +
                "    return redis.call(\"incr\", KEYS[1])\n" +
                "else\n" +
                "    return nil\n" +
                "end";
        DefaultRedisScript<Long> redisScript = new DefaultRedisScript<>();
        redisScript.setScriptText(incrIfExistScript);
        redisScript.setResultType(Long.class);
        Long nextMemberCodeNumber = redisHelper.execute4Long(redisScript, Collections.singletonList(key));
        if (nextMemberCodeNumber != null) {
            return MemberUserCodeUtils.generateMemberCode(nextMemberCodeNumber);
        }
        RLock lock = redissonClient.getLock("next:member:code:lock");
        try {
            boolean lockSuccess = lock.tryLock(2, 2, TimeUnit.SECONDS);
            if (!lockSuccess) {
                throw exception(GET_LOCK_FAILED);
            }
            nextMemberCodeNumber = redisHelper.execute4Long(redisScript, Collections.singletonList(key));
            if (nextMemberCodeNumber != null) {
                return MemberUserCodeUtils.generateMemberCode(nextMemberCodeNumber);
            }
            String currentMaxMemberCode = memberUserMapper.getCurrentMaxMemberCode();
            Long memberCodeMaxNumber = MemberUserCodeUtils.getMemberCodeNumber(currentMaxMemberCode);
            redisHelper.set(key, String.valueOf(memberCodeMaxNumber), 5, TimeUnit.MINUTES);
            return MemberUserCodeUtils.generateMemberCode(redisHelper.incrBy(key, 1));
        } catch (InterruptedException e) {
            throw exception(GET_LOCK_FAILED);
        } finally {
            lock.unlock();
        }
    }

}
